
/********************************************************************************
	Title:    clock.c
	Author:   Matthias Reis <mail@mreis.de>
				Kalenderrutinen von Herbert Vo
	Date:     27.04.2004
	Purpose:  Uhrroutine fr die GrossUhr, Zeit/Datumsgrenzen bercksichtigen
	Software: AVR-GCC with GNU binutils to compile, assemble and link needed
	Hardware: ATMega8 at 8 Mhz
********************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <inttypes.h> 
#include "Types.h"
#include "Main.h"
#include "clock.h"
#include "moon.h"

struct time clock;

u8 mTage[]={31,28,31,30,31,30,31,31,30,31,30,31};
u16 ztage[]={0,31,59,90,120,151,181,212,243,273,304,334};


/***************************************************************************************************

Function		:wochentag

Parameter In	: tag,monat,jahr
Parameter Out	: Wochentag

Description		: Berechnet den Wochentag zum aktuellen Datum

Date			: MR .04.2004

****************************************************************************************************/

u8 wochentag(u8 tag, u8 monat, u16 jahr)
{
	return ((dayDiff(1,1,1900,tag,monat,jahr))%7)+1;	
}

/***************************************************************************************************

Function		: istSchaltjahr

Parameter In	: jahr
Parameter Out	: 0-kein Schaltjahr, 1-Schaltjahr

Description		: Berechnet, ob das Jahr jahr ein Schaltjahr ist.

Date			: MR .04.2004

****************************************************************************************************/
u8 istSchaltjahr( u16 jahr)
{
   return (((jahr%4==0) && (jahr%100!=0))?1:(jahr%400==0)?1:0);
}

/***************************************************************************************************

Function		: anzSchaltJahre

Parameter In	: jahr
Parameter Out	: -

Description		: Berechnet, wieviele Schalttage seit dem 16 Jhr. vergangen sind

Date			: MR .04.2004

****************************************************************************************************/
u16 anzSchaltJahre(u16 jahr)
{
	return ((jahr-1597)/4 -(jahr-1501)/100 +(jahr-1201)/400);
}
	
/***************************************************************************************************

Function		: dayDiff

Parameter In	: 
Parameter Out	: -

Description		: Berechnet die Differenz zwischen 2 Datum in Tagen. Datum1<Datum2

Date			: MR .04.2004

****************************************************************************************************/
u16 dayDiff(u8 tag1, u8 monat1, u16 jahr1,u8 tag2, u8 monat2, u16 jahr2)
{
	u16 anztage;

	anztage=(jahr2-jahr1)*365;

	anztage+=anzSchaltJahre(jahr2+2000)-anzSchaltJahre(jahr1+2000);
	anztage+=ztage[monat2-1]+tag2;
	anztage-=ztage[monat1-1]+tag1;

	if (istSchaltjahr(jahr2) && monat2>2)
		anztage++;
	return anztage;
}

/***************************************************************************************************

Function		:setDay

Parameter In	: -
Parameter Out	: -

Description		: Berechnet den Wochentag zum aktuellen Datum

Date			: MR .04.2004

****************************************************************************************************/

void setDay(void)
{
	clock.wday=wochentag(clock.day,clock.month,clock.year+2000);
}

/***************************************************************************************************

Function		: incSec

Parameter In	: -
Parameter Out	: -

Description		: Erhht den struct clock um eine Sekunde

Date			: MR .04.2004
****************************************************************************************************/

void incSec(void)
{
	//Uhrzeit um 1s erhhen
	clock.second++;
	
	if (clock.second==60)
	{
		
		flClockRefresh=1;	//Signal an Main, die Zeit neu anzuzeigen
		clock.minute++;
		clock.second=0;
		
		if (clock.minute==60)	//Minutengrenze
		{
			clock.hour++;
			clock.minute=0;
			//Mondphase stndlich berechnen
			setMoon();
			if (clock.hour==24)	//Stundengrenze
			{
				clock.day++;
				clock.wday++;
				clock.hour=0;
				if (clock.wday==8)		//Wochengrenze
					clock.wday=1;
					
				if (clock.day==mTage[clock.month-1] + (istSchaltjahr(clock.year) & (clock.month==2 ? 2 : 1)))	//Monatsgrenze (Schaltjahr bercksichtigen)
				{
					clock.month++;
					clock.day=1;
					if (clock.month==13)
						clock.month=1;
				}
			}
		}
		
	}
}