/********************************************************************************
	Title:    ISR.c
	Author:   Matthias Reis <mail@mreis.de>, basiert auf den Routinen von Peter Danegger
	Date:     27.04.2004
	Purpose:  Dekodierung des DCF77-Signals
	Software: AVR-GCC with GNU binutils to compile, assemble and link needed
	Hardware: ATMega8 at 8 Mhz
********************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <inttypes.h> 
#include "Types.h"
#include "Main.h"
#include "Display.h"
#include "ISR.h"
#include "dcf77.h"
#include "clock.h"


u8 dcf77_period;		//DCF77-Pause
u8 dcf77_pulse;			//DCF77-Pulse

struct time newtime;
u8 dcf77error=1;		//Erst versuchen, einen Minutenbeginn zu erkennen

/***************************************************************************************************

Function		: decode_dcf77

Parameter In	: Bit, das gerade empfangen wurde
Parameter Out	: -

Description		: Decodiert den dcf77-Stream

Date			: PD ??.??.????
				  MR 27.04.2004 Portiert auf GCC

****************************************************************************************************/

void decode_dcf77(u8 pulse )
{
	static u8 parity;
	u8 BITNO[] = { 
						0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06,		// minute
						0xFF,											// parity
						0x10, 0x11, 0x12, 0x13, 0x14, 0x15,				// hour
						0xFF,											// parity
						0x20, 0x21, 0x22, 0x23, 0x24, 0x25,				// hour
						0x30, 0x31, 0x32,								// weekday
						0x40, 0x41, 0x42, 0x43, 0x44,					// month
						0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57,	// year
						0xFF };											// parity
	u8 BMASK[] = { 1, 2, 4, 8, 10, 20, 40, 80 };
	u8 i;
	u8 *d;

	i = newtime.second - 21;
	if( i >= sizeof( BITNO ))					// only bit 21 ... 58
		return;
		
	parity ^= pulse;							// calculate parity
	i = BITNO[i];
	if( i == 0xFF )
	{											// test parity
		if( parity )
			dcf77error = 1;
		parity = 0;
		return;
	}
	d = (u8 *) & newtime.minute + (i >> 4);	// byte address
	i &= 0x0F;									// bit number
	
	if( i == 0 )
		*d = 0;									// clear all, if lsb
	
	if( pulse )
		*d += BMASK[i];						// set bit
}


/***************************************************************************************************

Function		: scan_dcf77

Parameter In	: -
Parameter Out	: -

Description		: Beurteilt, ob Pause, 1, 0 oder Minutenanfang gesendet wurde

Date			: PD ??.??.????
				  MR 27.04.2004 Portiert auf GCC
				  
****************************************************************************************************/

void scan_dcf77( void )
{
	if(dcf77_pulse)	//Pulse aufgetreten
	{
		if( dcf77_pulse >DCF77_BITCLR_LO  && dcf77_pulse < DCF77_BITCLR_HI )	//0 (100ms) erkannt?
		{
			decode_dcf77(0);
		}
		else
		{
			if( dcf77_pulse > DCF77_BITSET_LO && dcf77_pulse < DCF77_BITSET_HI )	//1 (200ms) erkannt?
			{
				decode_dcf77(1);
			}
			else
			{
				dcf77error = 1;
			}
		}
		
		dcf77_pulse = 0;
	}

	if(dcf77_period)	//Pause aufgetreten
	{
		if(newtime.second < 60)
			newtime.second++;
		  
		if(dcf77_period > DCF77_PERIOD_LO && dcf77_period < DCF77_PERIOD_HI)	//Minutenbeginn erkannt?
		{
			if(dcf77error == 0 && newtime.second ==59)
			{
				newtime.second = 0;
				//sync_sec();
				newtime.moon=clock.moon;	//Mondstellung retten!
				clock = newtime;
				flClockRefresh=1;			//Signal an Main, die Zeit neu anzuzeigen
				flInit=TRUE;				//Initialisierung durchgefhrt
				flBlink=FALSE;
			}
			newtime.second = 0;
			dcf77error = 0;
		}
		else
		{
			if( dcf77_period < DCF77_ERROR_LO || dcf77_period > DCF77_ERROR_HI )	//Alles auerhalb einer Sekunde mu ein Fehler sein
				dcf77error = 1;
		}
		
		dcf77_period = 0;
	}
}
