/********************************************************************************
	Title:    Display.c
	Author:   Matthias Reis <mail@mreis.de>
	Date:     27.04.2004
	Purpose:  AnzeigeRoutinen fr die GrossUhr
	Software: AVR-GCC with GNU binutils to compile, assemble and link needed
	Hardware: ATMega8 at 8 Mhz
********************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <inttypes.h> 
#include "Types.h"
#include "Display.h"
#include "clock.h"
#include "moon.h"
#include "main.h"
#include "ADC.h"
#include "dcf77.h"

u8 tempDisplay[3];		//Temperatur
u8 clockDisplay[10];	//Uhrzeit und Datum + Mond

/*
bersetzungstabelle fr 7seg
	
	   _____
	  |  a  |
	 f|     |b
	  |_____|
	  |  g  |
	 e|     |c
	  |_____|
		 d
	
	
	Anschlsse:
	HC595	Displ.Segment
	QA	=	d
	QB	=	e
	QC	=	f
	QD	=	g
	QE	=	a
	QF	=	b
	QG	=	c
	QH	=	DF

bersetzung:
	Ziffer 	Display		HC595
	 0		111 1110 0	1110 111 0	= 0xEE	
	 1		011 0000 0	0000 011 0	= 0x06
	 2		110 1101 0	1101 110 0	= 0xDC
	 3		111 1001 0	1001 111 0	= 0x9E
	 4		011 0011 0	0011 011 0	= 0x36
	 5		101 1011 0	1011 101 0	= 0xBA
	 6		101 1111 0	1111 101 0	= 0xFA
	 7		111 0000 0	0000 111 0	= 0x0E
	 8		111 1111 0	1111 111 0	= 0xFE
	 9		111 1011 0	1011 111 0	= 0xBE
	DF		000 0000 1	0000 000 1	= 0x01
	Clear	000 0000 0  0000 000 0  = 0x00

*/
u8 seg7[]={0xee,0x06,0xdc,0x9e,0x36,0xba,0xfa,0x0e,0xfe,0xbe,0x00};	//Umwandlung von Zahl in 7seg-Darstellung

//Fonttabelle fr die Wochentagsanzeige
u8 wdays[]={	0x00,0x3a,0x00,0x06,0x16,0x06,	//MO
				0x00,0x1c,0x22,0x06,0x3e,0x3e,	//Di
				0x00,0x3a,0x00,0x06,0x3e,0x3e, 	//Mi
				0x00,0x1c,0x22,0x06,0x16,0x06, 	//Do
				0x00,0x34,0x3c,0x06,0x36,0x3e,	//Fr
				0x10,0x14,0x04,0x2e,0x16,0x06,	//Sa
				0x10,0x14,0x04,0x06,0x16,0x06, 	//So
				0x22,0x1c,0x2a,0x00,0x1e,0x1e, 	//Cl
				0x00,0x1c,0x22,0xff,0xff,0xff 	//D
				};

/***************************************************************************************************

Function		: WDay

Parameter In	: in: Wochentag, 0-MO...6-SO
Parameter Out	: -

Description		: 

Date			: MR .04.2004

****************************************************************************************************/

void WDayInt(void)
{
	//cbi(PORTD,G_D);
	static u8 RowCount=0,start=0,Takt=0;
	u8 i;
	
	//Was soll angezeigt werden?
	if (!gFullDisplay)
	{
		PORTB=0x3e;					//LEDs aus	
		return;
	}

	if (Takt>0)
	{
		Takt--;
		return;
	}
	
	Takt=LDR;
	
	//for (j=0;j<1;j++);
	
	if (RowCount==0)	//Alles initialisieren
	{
		start=((clock.wday-1)*6)+6;
		
		//Eine 1 erzeugen
		cbi(PORTD,CLK_D);			//Clock-Lo
		sbi(PORTB,SI);				//Set SI
		sbi(PORTD,CLK_D);			//Clock-Hi

		//QA und QB sind nicht beschaltet
		for (i=0;i<2;i++)
		{
			cbi(PORTD,CLK_D);		//Clock-Lo
			cbi(PORTB,SI);			//Set SI
			sbi(PORTD,CLK_D);		//Clock-Hi
		}
	}
	
	PORTB=0x3e;						//LEDs aus
	cbi(PORTD,CLK_D);				//Clock-Lo
	cbi(PORTB,SI); 					//Clear SI
	sbi(PORTD,CLK_D);				//Clock-Hi, SI bernehmen
	PORTB=wdays[--start];			//LEDs an
	
	//delay(0);
	RowCount++;
	
	if (RowCount==7)
	{
		//letztes Bit um CLK und RCK zu synchronisieren
		cbi(PORTD,CLK_D);			//Clock-Lo
		cbi(PORTB,SI);				//Set SI
		sbi(PORTD,CLK_D);			//Clock-Hi
		RowCount=0;
	}
}

/***************************************************************************************************

Function		: finalizeTemp

Parameter In	: -
Parameter Out	: -

Description		: Sendet das letzte Bit um RCK und SCK zu synchroniesieren

Date			: MR 23.04.2004

****************************************************************************************************/
void finalizeTemp(void)
{
	cbi(PORTD,CLK_T);		
	sbi(PORTD,CLK_T);		//Clock-Hi, SI bernehmen
	cbi(PORTD,CLK_T);		
}
/***************************************************************************************************

Function		: finalizeClock

Parameter In	: -
Parameter Out	: -

Description		: Sendet das letzte Bit um RCK und SCK zu synchroniesieren

Date			: MR 23.04.2004

****************************************************************************************************/
void finalizeClock(void)
{
	cbi(PORTD,CLK_C);		
	sbi(PORTD,CLK_C);		//Clock-Hi, SI bernehmen
	cbi(PORTD,CLK_C);		
}

/***************************************************************************************************

Function		: 

Parameter In	: -
Parameter Out	: -

Description		: 

Date			: MR .04.2004

****************************************************************************************************/
void SendByteToTemp(u8 in)
{
	u8 i;
	
	for (i=0;i<8;i++)
	{
		cbi(PORTD,CLK_T);		//Clock-Lo
		if ((in >> i) & 1)
			sbi(PORTB,SI);		//Set SI
		else
			cbi(PORTB,SI);		//Clear SI
		sbi(PORTD,CLK_T);		//Clock-Hi, SI bernehmen
	}	
	cbi(PORTD,CLK_T);		//Clock-Hi
}
/***************************************************************************************************

Function		: 

Parameter In	: -
Parameter Out	: -

Description		: Schaltet Display aus

Date			: MR .04.2004

****************************************************************************************************/
void SwitchOff(void)
{
	cli();			//Keine Interrrupts mehr wg. Timer0
	sbi(PORTD,G);
}
/***************************************************************************************************

Function		: 

Parameter In	: -
Parameter Out	: -

Description		: Schaltet Display an

Date			: MR .04.2004

****************************************************************************************************/
void SwitchOn(void)
{
	cbi(PORTD,G);
	sei();			//interrupts wieder erlauben
}

/***************************************************************************************************

Function		: 

Parameter In	: -
Parameter Out	: -

Description		: 

Date			: MR .04.2004

****************************************************************************************************/

void SendByteToClock(u8 in)
{
	u8 i;
	for (i=0;i<8;i++)
	{
		cbi(PORTD,CLK_C);		//Clock-Hi
		if ((in >> i) & 1)
			sbi(PORTB,SI);		//Set SI
		else
			cbi(PORTB,SI);		//Clear SI
		sbi(PORTD,CLK_C);		//Clock-Low, SI bernehmen
	}
	cbi(PORTD,CLK_C);		//Clock-Lo
}

/***************************************************************************************************

Function		: 

Parameter In	: -
Parameter Out	: -

Description		: 

Date			: MR .04.2004

****************************************************************************************************/

void EraClock(void)
{
	u8 i;
	SwitchOff();
	for (i=0;i<10;i++)
	{
		SendByteToClock(0);
	}
	finalizeClock();
	SwitchOn();
}
/***************************************************************************************************

Function		: 

Parameter In	: -
Parameter Out	: -

Description		: 

Date			: MR .04.2004

****************************************************************************************************/

void EraTemp(void)
{
	u8 i;
	SwitchOff();
	for (i=0;i<3;i++)
	{
		tempDisplay[i]=ERA_DISPL;
		SendByteToTemp(0);
	}
	finalizeTemp();
	SwitchOn();
}

/***************************************************************************************************

Function		: numTo7seg

Parameter In	: in - Darzustellende Zahl
				  df - Dezimalpunkt an (1) oder aus (0)
Parameter Out	: -

Description		: Wandelt eine Zahl in 7Seg um

Date			: MR 23.04.2004

****************************************************************************************************/

u8 numTo7seg(u8 in, BOOL df)
{
	return df ? seg7[in]|0x01 : seg7[in];
}

/***************************************************************************************************

Function		: setTemp

Parameter In	: in - Darzustellende Zahl
Parameter Out	: -

Description		: Trgt das Zeichen in auf Position pos ein

Date			: MR 23.04.2004

****************************************************************************************************/
void setTemp(i16 in, BOOL df)
{
	u8 i;
	BOOL df2;
	
	//Was soll angezeigt werden?
	if (!gFullDisplay)
	{
		for (i=0;i<3;i++)
		{
			tempDisplay[i]=numTo7seg(ERA_DISPL,FALSE);
		}
		return;
	}
	
	for (i=0;i<3;i++)
	{	
		df2=FALSE;

		if (i==0)
			df2=TRUE;
		
		if (i==1 && df && (in>=0))
			df2=TRUE;
			
		if (i==2 && in<0)
			df2=TRUE;
		
		if (i==2 && df)
			df2=TRUE;
				
		tempDisplay[i]=numTo7seg((in<0?in*-1:in) % 10,df2); 	//Die letzte Stelle extrahieren
		in=in/10;
	}
}

/***************************************************************************************************

Function		: displTemp

Parameter In	: -
Parameter Out	: -

Description		: Gibt das Array tempDispl[] auf dem Temperatur-Display aus

Date			: MR 23.04.2004

****************************************************************************************************/
void displTemp(void)
{
	u8 i;
	
	SwitchOff();
	for (i=0;i<3;i++)
	{
		SendByteToTemp(tempDisplay[i]);
	}
	finalizeTemp();
	SwitchOn();
}
/***************************************************************************************************

Function		: displClock

Parameter In	: -
Parameter Out	: -

Description		: Gibt das Array tempClock[] auf dem Clock-Display aus

Date			: MR 23.04.2004

****************************************************************************************************/
void displClock(void)
{
	u8 i;
	
	SwitchOff();
	for (i=0;i<10;i++)
	{
		SendByteToClock(clockDisplay[i]);
	}
	finalizeClock();
	SwitchOn();
}
    
/***************************************************************************************************

Function		: numToMoon

Parameter In	: Mondstand
Parameter Out	: Byte 1,2 fr HC595's in der Mondanzeige

Description		: Umrechnung des Mondstandes (0-21) auf die Anzeige

Date			: MR 23.04.2004

****************************************************************************************************/

void numToMoon(u8 in, u8 *num1, u8 *num2)
{
	u8 shift=in;
	//moon ist zw.0-21
	u16 tmp=0xffff;
	
	if (in>11)
		shift=in-11;	//Abnehmender Mond
			
	tmp = tmp >> shift;	//0-11 Bits nach rechts schieben
	if (in<12)
		tmp = ~tmp;		//Umdrehen
	tmp = tmp >>3;		//Noch 3 Bits nach rechts (wg. HW)
	*num1=tmp /*& 0xff*/;			//Die unteren 8 Bit nach num1
	*num1=*num1>>1;		//Noch ein Bit weiter (wg.HW)
	tmp=tmp>>8;			//Die oberen Bit nach unten
	*num2=tmp;			
}

    
/***************************************************************************************************

Function		: Umwandlung einer u8 (2stellig) zu 2 Ziffern

Parameter In	: in - u8-Zahl
Parameter Out	: num1/num2 - 1/2 Stelle

Description		: Extrahiert aus einer Zahl 2 Stellen.

Date			: MR 23.04.2004

****************************************************************************************************/
void BCD(u8 in, u8 *num1, u8 *num2)
{
	*num1=in%10;
	*num2=in/10;
}

/***************************************************************************************************

Function		: setClock

Parameter In	: -
Parameter Out	: -

Description		: Trgt die Uhrzeit/Datum/Mondstellung in das Array clockDisplay ein

Date			: MR 23.04.2004

****************************************************************************************************/
void setClock(void)
{
	u8 i=0,n1,n2;

	SwitchOff();
	
	//Anzeige Monat
	if (gFullDisplay)
		BCD(clock.month,&n1,&n2);
	else
		n1=n2=ERA_DISPL;
		
	clockDisplay[i]=numTo7seg(n1,gFullDisplay);
	clockDisplay[++i]=numTo7seg(n2,gFullDisplay);
	
	//Anzeige Tag
	if (gFullDisplay)
		BCD(clock.day,&n1,&n2);
	else
		n1=n2=ERA_DISPL;
	clockDisplay[++i]=numTo7seg(n1,gFullDisplay);
	clockDisplay[++i]=numTo7seg(n2,gFullDisplay);
	
	//Anzeige Minute
	BCD(clock.minute,&n1,&n2);
	clockDisplay[++i]=numTo7seg(n1,!dcf77error);	//Wenn kein dcf77 Empfang soll der Doppelpunkt (00:00) ausgehen
	clockDisplay[++i]=numTo7seg(n2,!dcf77error);
	
	//Anzeige Stunde
	BCD(clock.hour,&n1,&n2);
	clockDisplay[++i]=numTo7seg(n1,TRUE);
	clockDisplay[++i]=numTo7seg(n2,TRUE);
	
	//Anzeige Mond
	if (gFullDisplay)
		numToMoon(clock.moon,&n1,&n2);
	else
		n1=n2=0;
	clockDisplay[++i]=n1;	//Linke Mondhlfte
	clockDisplay[++i]=n2;	//Rechte Mondhlfte
	
	finalizeClock();
	SwitchOn();
}
