/********************************************************************************
	Title:    ISR.c
	Author:   Matthias Reis <mail@mreis.de>
	Date:     27.04.2004
	Purpose:  InterruptRoutinen fr die GrossUhr
	Software: AVR-GCC with GNU binutils to compile, assemble and link needed
	Hardware: ATMega8 at 8 Mhz
********************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <inttypes.h> 
#include "Types.h"
#include "Main.h"
#include "Display.h"
#include "ISR.h"
#include "adc.h"
#include "dcf77.h"
#include "Keys.h"


/***************************************************************************************************

Function		: SIG_ADC

Parameter In	: in: Wochentag, 0-MO...6-SO
Parameter Out	: -

Description		: 

Date			: MR .04.2004

****************************************************************************************************/
SIGNAL(SIG_ADC)
{
	switch (inMux)
	{
		case 1:
			LDR=(LDR+(ADC/4))/2;
			break;
		case 2:
			NTC1=(NTC1+ADC)/2;
			break;
		case 3:
			NTC2=(NTC2+ADC)/2;
			break;
	}
}

/***************************************************************************************************

Function		: WDay

Parameter In	: in: Wochentag, 0-MO...6-SO
Parameter Out	: -

Description		: 

Date			: MR .04.2004

****************************************************************************************************/
SIGNAL(SIG_OVERFLOW2)
{
	
	//TCNT2 = QUARZ;			//Timer neu laden
	flADCRead=TRUE;				//Sensoren auslesen
	
	if (!flInit)				//Keine Initialisierung durchgefhrt
	{
		flBlink=!flBlink;
	}	
	else
		flOneSec=TRUE;			//Eine Sekunde ist vergangen
	
	if (gTimeout)
		gTimeout--;
		
	gTempSwitch++;
	
}

/***************************************************************************************************

Function		: SIG_OVERFLOW0

Parameter In	: -
Parameter Out	: -

Description		: Helligkeitssteuerung, DCF77 Signal abfragen

Date			: MR .04.2004

****************************************************************************************************/
SIGNAL(SIG_OVERFLOW0)
{
	static u8 dcf77_slow=0;	//"Verlangsamt" die Abfrage auf ca. 6ms
	static u8 old_dcf77=0;
	static u8 t=0;			//Anzahl Takte
	

	TCNT0 = COUNTER;				//Timer neu laden	
	
	if (!flBlink)					//Bei nicht initialisierter Uhr blinken
	{
		if (bright<=LDR+bright_hi)			//Hellphase
		{
			cbi(PORTD,G);
			cbi(PORTD,G_D);
		}
		else						//Dunkelphase
		{
			sbi(PORTD,G);
			//sbi(PORTD,G_D);		//Dunkelphase fr Multiplexed Wochentagsanzeige
		}
	}
	else	//Dunkel schalten
	{
			sbi(PORTD,G);
			sbi(PORTD,G_D);		//Dunkelphase fr Multiplexed Wochentagsanzeige
	}
	
	bright++;
	if (bright>BRIGHT_HI)
		bright=BRIGHT_LO;
	
	//DCF77Signal erfassen
	if (--dcf77_slow)				//Erst nach 255 Takten (~6ms) DCF77-Signal abtasten
		return;
		
	if (t)
		t++;
		
	if (bit_is_set(PIND,DCF77))	//Abfrage des Ports. DCF77 nicht invertiert.
	{
		if( old_dcf77 == 1 )		//Wenn 1, dann ist das Signal t Takte lang 0 gewesen.
		{
		  old_dcf77 = 0;
		  dcf77_period = t;
		  t = 1;
		}
	}
	else
	{
		if( old_dcf77 == 0 )		//Wenn 0, dann ist das Signal t Takte lang 1 gewesen.
		{
		  old_dcf77 = 1;
		  dcf77_pulse = t;
		}
	}
	
	//Tastenrepeat
	if (flKeyLatency)
		flKeyLatency--;
	
}
