/********************************************************************************
	Title:    Init.c
	Author:   Matthias Reis <mail@mreis.de>
	Date:     27.04.2004
	Purpose:  Initialisierung fr die GrossUhr
	Software: AVR-GCC with GNU binutils to compile, assemble and link needed
	Hardware: ATMega8 at 8 Mhz
********************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <inttypes.h>
#include "Types.h"
#include "Main.h"
#include "Init.h"

/***************************************************************************************************

Function		: Init

Parameter In	: -
Parameter Out	: -

Description		: Initialisieren der Ports und der Variablen

Date			: MR 23.04.2004

****************************************************************************************************/
void init(void)
{
	//Initialisieren der Ports, AD-Wandler etc.
	
	//Ports
	DDRB=0xff;								//Port B Als Output
	PORTB=0;								//Port lschen
	
	DDRC=0;									//Port C als Input
	PORTC=0xf8;								//Pull-Ups aktivieren fr PC3-PC7, Rest ist ADC
											//PD0,1,2,6,7=OUT, PD3,4,5=IN 
	DDRD=_BV(DDD0) | _BV(DDD1) | _BV(DDD2)| _BV(DDD6) | _BV(DDD7);
											//Pull-Ups fr PD3,4,5 aktivieren, Rest lschen
	PORTD=_BV(PIND3) | _BV(PIND4) | _BV(PIND5);	
	
	//ADC
	ADMUX=0x00;								//Channel 1, Ref=VCC
											//Enable Int, Prescale=64
	ADCSRA =  _BV(ADEN) | _BV(ADPS2) | _BV(ADPS1) | _BV(ADIE);	

	//Timer
	TIMSK=0;
	//Timer 0
	TCNT0 = COUNTER;	      				//Rcksetzen des Timers
	TCCR0 = _BV(CS01);    					//Prescaler 8
	
	//Timer 2 - Clock Crystal 32.768 Hz
	ASSR |= _BV(AS2);						//Clock Crystal Connected to TOSC1/TOSC2 (CKOPT- Fuse Set to 1) 
//	TCNT2=QUARZ;
	OCR2=0;
	TCCR2=_BV(CS22) | _BV(CS20);			//Prescaler=128 (*256=1 sec)
	
	TIFR |=_BV(TOV2) | _BV(TOV0); 			//Int lschen
	TIMSK = _BV(TOIE0) | _BV(TOIE2);     	//Timer0/2 Overflow Interrupt enable, 
	
	TCCR1A=0;								//Disable PWM and Compare Output Mode
	
}